from unittest import TestCase

from sister.word_embedders import FasttextEmbedding


class FasttextEmbeddingCase(TestCase):

    def setUp(self):
        self.embedding = FasttextEmbedding(lang="en")

    def tearDown(self):
        pass

    def test_get_word_vector(self):
        word = 'test'
        vector = self.embedding.get_word_vector(word)
        self.assertEqual(vector.shape, (300,))

    def test_get_word_vectors(self):
        words = ['good', 'test']
        vectors = self.embedding.get_word_vectors(words)
        self.assertEqual(vectors.shape, (len(words), 300))



from unittest import TestCase

from sister import tokenizers


class SimpleTokenizerCase(TestCase):

    def setUp(self):
        self.tokenizer = tokenizers.SimpleTokenizer()

    def tearDown(self):
        pass

    def test_tokenize(self):
        sentence = 'I would like to visit Japan.'
        gold = [
                'I',
                'would',
                'like',
                'to',
                'visit',
                'Japan',
                '.'
                ]
        tokens = self.tokenizer.tokenize(sentence)
        self.assertEqual(gold, tokens)



from typing import List
from pathlib import Path

from fasttext import load_model
import numpy as np

from sister.download import cached_download, cached_unzip


def get_fasttext(lang: str = "en"):
    # Download.
    urls = {
            "en": "https://dl.fbaipublicfiles.com/fasttext/vectors-wiki/wiki.simple.zip"
            }
    path = cached_download(urls[lang])
    path = Path(path)
    dirpath = path.parent / 'fasttext'
    cached_unzip(path, dirpath)

    print("Loading model...")
    model = load_model(str(dirpath / 'wiki.simple.bin'))
    return model


class WordEmbedding(object):

    def get_word_vector(self, word: str) -> np.ndarray:
        raise NotImplementedError

    def get_word_vectors(self, words: List[str]) -> np.ndarray:
        raise NotImplementedError


class FasttextEmbedding(WordEmbedding):

    def __init__(self, lang: str = "en") -> None:
        model = get_fasttext(lang)
        self.model = model

    def get_word_vector(self, word: str) -> np.ndarray:
        return self.model.get_word_vector(word)

    def get_word_vectors(self, words: List[str]) -> np.ndarray:
        vectors = []
        for word in words:
            vectors.append(self.get_word_vector(word))
        return np.array(vectors)



from typing import List


class Tokenizer(object):

    def tokenize(self, sentence: str) -> List[str]:
        raise NotImplementedError


class SimpleTokenizer(Tokenizer):

    def __init__(self):
        self.replace_tokens = [
                ('.', ' .'),
                ('?', ' ?'),
                ('!', ' !'),
                ('(', ' ( '),
                (')', ' ) '),
                ]

    def tokenize(self, sentence: str) -> List[str]:
        for replace_token in self.replace_tokens:
            sentence = sentence.replace(*replace_token)
        return sentence.split()



import tempfile
import contextlib
import hashlib
import os
import shutil
import sys
import zipfile
from urllib import request
from pathlib import Path


_cache_root = os.environ.get(
    'SISTER_CACHE_ROOT',
    os.path.join(os.path.expanduser('~'), '.sister', 'cache'))


@contextlib.contextmanager
def tempdir(**kwargs):
    ignore_errors = kwargs.pop('ignore_errors', False)

    temp_dir = tempfile.mkdtemp(**kwargs)
    try:
        yield temp_dir
    finally:
        shutil.rmtree(temp_dir, ignore_errors=ignore_errors)


def get_cache_root() -> str:
    return _cache_root


def set_cache_root(path: str) -> None:
    global _cache_root
    _cache_root = path


def get_cache_directory(cache_name: str,
                        create_directory: bool = True) -> str:
    path = os.path.join(_cache_root, cache_name)
    if create_directory:
        try:
            os.makedirs(path)
        except OSError:
            if not os.path.isdir(path):
                raise
    return path


def cached_download(url: str) -> str:
    cache_root = os.path.join(_cache_root, '_dl_cache')
    try:
        os.makedirs(cache_root)
    except OSError:
        if not os.path.isdir(cache_root):
            raise

    urlhash = hashlib.md5(url.encode('utf-8')).hexdigest()
    cache_path = os.path.join(cache_root, urlhash)

    if os.path.exists(cache_path):
        return cache_path

    with tempdir(dir=cache_root) as temp_root:
        temp_path = os.path.join(temp_root, 'dl')
        sys.stderr.write('Downloading from {}...\n'.format(url))
        sys.stderr.flush()
        request.urlretrieve(url, temp_path)
        shutil.move(temp_path, cache_path)

    return cache_path


def cached_unzip(path: Path, saveto: Path) -> None:
    if not saveto.exists():
        print('Unzipping...')
        with zipfile.ZipFile(path, 'r') as f:
            f.extractall(saveto)



from unittest import TestCase

from sister import MeanEmbedding


class MeanEmbeddingCase(TestCase):

    def setUp(self):
        self.sentence_embedding = MeanEmbedding()
        pass

    def tearDown(self):
        pass

    def test_embed(self):
        sentence = "I am a dog."
        vector = self.sentence_embedding.embed(sentence)
        self.assertEqual(vector.shape, (300,))



import numpy as np

from sister.tokenizers import Tokenizer, SimpleTokenizer
from sister.word_embedders import WordEmbedding, FasttextEmbedding


class SentenceEmbedding(object):

    def __init__(
            self,
            tokenizer: Tokenizer,
            word_embedder: WordEmbedding) -> None:
        self.tokenizer = tokenizer
        self.word_embedder = word_embedder

    def embed(self, sentence: str) -> np.ndarray:
        raise NotImplementedError

    def __call__(self, sentence: str) -> np.ndarray:
        raise NotImplementedError


class MeanEmbedding(SentenceEmbedding):

    def __init__(
            self,
            tokenizer: Tokenizer = SimpleTokenizer(),
            word_embedder: WordEmbedding = FasttextEmbedding()) -> None:
        self.tokenizer = tokenizer
        self.word_embedder = word_embedder

    def embed(self, sentence: str) -> np.ndarray:
        tokens = self.tokenizer.tokenize(sentence)
        vectors = self.word_embedder.get_word_vectors(tokens)
        return np.mean(vectors, axis=0)

    def __call__(self, sentence: str) -> np.ndarray:
        return self.embed(sentence)



#!/usr/bin/env python
# coding: utf-8
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


setup(
        name='sister',
        use_scm_version=True,
        setup_requires=['setuptools_scm'],
        description='SImple SenTence EmbeddeR',
        long_description=open('./README.md', encoding='utf-8').read(),
        long_description_content_type='text/markdown',
        url='https://github.com/sobamchan/sister',
        author='Sotaro Takeshita',
        author_email='oh.sore.sore.soutarou@gmail.com',
        packages=[
            'sister'
            ],
        license='MIT',
        classifiers=[
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7'
            ],
        install_requires=['fasttext', 'numpy'],
        tests_requires=['pytest']
        )
